//
//  call-remote-view.swift
//  adamo-sample
//
//  Created by Trần Tý on 8/13/19.
//  Copyright © 2019 Trần Tý. All rights reserved.
//

//
//  UIView+Simulate.swift
//  adamo-sample
//
//  Created by Trần Tý on 8/12/19.
//  Copyright © 2019 Trần Tý. All rights reserved.
//

import TwilioVideo
// Helper to determine if we're running on simulator or device

class CallRemoteView: UIView {
    
    var preview: TVIVideoView = TVIVideoView()
    
    //  MARK: DATA
    var participant: TVIRemoteParticipant!
    
    
    // MARK: EVENT
    
    @objc var onPublishedVideoTrack: RCTDirectEventBlock!
    @objc var onUnPublishedVideoTrack: RCTDirectEventBlock!
    @objc var onPublishedAudioTrack: RCTDirectEventBlock!
    @objc var onUnPublishedAudioTrack: RCTDirectEventBlock!
    @objc var onSubcribeVideoTrack: RCTDirectEventBlock!
    @objc var onUnSubcribeVideoTrack: RCTDirectEventBlock!
    @objc var onSubcribeAudioTrack: RCTDirectEventBlock!
    @objc var onUnSubcribeAudioTrack: RCTDirectEventBlock!
    
    @objc var onEnabledVideoTrack: RCTDirectEventBlock!
    @objc var onDisabledVideoTrack: RCTDirectEventBlock!
    @objc var onEnabledAudioTrack: RCTDirectEventBlock!
    @objc var onDisabledAudioTrack: RCTDirectEventBlock!
    
    @objc var onFailedToSubscribeVideoTrack: RCTDirectEventBlock!
    @objc var onFailedToSubscribeAudioTrack: RCTDirectEventBlock!
    
    required init?(coder aDecoder: NSCoder) {
        fatalError()
    }
    
    override init(frame: CGRect) {
        super.init(frame: frame)
        self.addSubview(preview)
        preview.delegate = self
        preview.fixInView(self)
    }
    
    func configureWihtParticipant(_ identity: String, callRoom: CallView){
        let p = callRoom.remoteParticipants.first(where: { (par) -> Bool in
            return par.identity == identity
        })
        self.participant = p
        participant.delegate = self
        if let track = participant.videoTracks.first?.videoTrack {
            track.addRenderer(preview)
        }
    }
    
}
extension CallRemoteView {
    func addRenderer(track: TVIRemoteVideoTrack){
        track.addRenderer(self.preview)
    }
    func removeRenderer(track: TVIRemoteVideoTrack){
        track.removeRenderer(preview)
    }
}
extension CallRemoteView: TVIVideoViewDelegate {
    func videoView(_ view: TVIVideoView, videoDimensionsDidChange dimensions: CMVideoDimensions) {
        self.setNeedsLayout()
    }
}

// MARK: TVIRemoteParticipantDelegate
extension CallRemoteView : TVIRemoteParticipantDelegate {
    func logMessage(messageText: String){
        print(messageText)
    }
    func remoteParticipant(_ participant: TVIRemoteParticipant,
                           publishedVideoTrack publication: TVIRemoteVideoTrackPublication) {
        UIView.animate(withDuration: 0.3, delay: 0, usingSpringWithDamping: 0.5, initialSpringVelocity: 0, options: [.curveEaseInOut], animations: {
            self.preview.transform = .identity
            self.preview.layer.opacity = 1
        }, completion: nil)
        onPublishedVideoTrack([:])
        // Remote Participant has offered to share the video Track.
        logMessage(messageText: "Participant \(participant.identity) published video track")
    }
    
    func remoteParticipant(_ participant: TVIRemoteParticipant,
                           unpublishedVideoTrack publication: TVIRemoteVideoTrackPublication) {
        
        UIView.animate(withDuration: 0.3, delay: 0, usingSpringWithDamping: 0.5, initialSpringVelocity: 0, options: [.curveEaseInOut], animations: {
            self.preview.transform = CGAffineTransform(scaleX: 0.1, y: 0.1)
            self.preview.layer.opacity = 0
        }, completion: nil)
        onUnPublishedVideoTrack([:])
        logMessage(messageText: "Participant \(participant.identity) unpublished video track")
    }
    
    func remoteParticipant(_ participant: TVIRemoteParticipant,
                           publishedAudioTrack publication: TVIRemoteAudioTrackPublication) {
        
        // Remote Participant has offered to share the audio Track.
        onPublishedAudioTrack([:])
        logMessage(messageText: "Participant \(participant.identity) published audio track")
    }
    
    func remoteParticipant(_ participant: TVIRemoteParticipant,
                           unpublishedAudioTrack publication: TVIRemoteAudioTrackPublication) {
        
        // Remote Participant has stopped sharing the audio Track.
        onUnPublishedAudioTrack([:])
        logMessage(messageText: "Participant \(participant.identity) unpublished audio track")
    }
    
    func subscribed(to videoTrack: TVIRemoteVideoTrack,
                    publication: TVIRemoteVideoTrackPublication,
                    for participant: TVIRemoteParticipant) {
        
        // We are subscribed to the remote Participant's video Track. We will start receiving the
        // remote Participant's video frames now.
        onSubcribeVideoTrack([:])
        videoTrack.addRenderer(self.preview)
        logMessage(messageText: "Subscribed to video track for Participant \(participant.identity)")
        
    }
    
    func unsubscribed(from videoTrack: TVIRemoteVideoTrack,
                      publication: TVIRemoteVideoTrackPublication,
                      for participant: TVIRemoteParticipant) {
        
        // We are unsubscribed from the remote Participant's video Track. We will no longer receive the
        // remote Participant's video.
        videoTrack.removeRenderer(self.preview)
        onUnSubcribeVideoTrack([:])
        logMessage(messageText: "Unsubscribed from video track for Participant \(participant.identity)")
        
    }
    
    func subscribed(to audioTrack: TVIRemoteAudioTrack,
                    publication: TVIRemoteAudioTrackPublication,
                    for participant: TVIRemoteParticipant) {
        
        // We are subscribed to the remote Participant's audio Track. We will start receiving the
        // remote Participant's audio now.
        onSubcribeAudioTrack([:])
        logMessage(messageText: "Subscribed to audio track for Participant \(participant.identity)")
    }
    
    func unsubscribed(from audioTrack: TVIRemoteAudioTrack,
                      publication: TVIRemoteAudioTrackPublication,
                      for participant: TVIRemoteParticipant) {
        
        // We are unsubscribed from the remote Participant's audio Track. We will no longer receive the
        // remote Participant's audio.
        onUnSubcribeAudioTrack([:])
        logMessage(messageText: "Unsubscribed from audio track for Participant \(participant.identity)")
    }
    
    func remoteParticipant(_ participant: TVIRemoteParticipant,
                           enabledVideoTrack publication: TVIRemoteVideoTrackPublication) {
        onEnabledVideoTrack([:])
        logMessage(messageText: "Participant \(participant.identity) enabled video track")
    }
    
    func remoteParticipant(_ participant: TVIRemoteParticipant,
                           disabledVideoTrack publication: TVIRemoteVideoTrackPublication) {
        logMessage(messageText: "Participant \(participant.identity) disabled video track")
        onDisabledVideoTrack([:])
    }
    
    func remoteParticipant(_ participant: TVIRemoteParticipant,
                           enabledAudioTrack publication: TVIRemoteAudioTrackPublication) {
        logMessage(messageText: "Participant \(participant.identity) enabled audio track")
        onEnabledVideoTrack([:])
    }
    
    func remoteParticipant(_ participant: TVIRemoteParticipant,
                           disabledAudioTrack publication: TVIRemoteAudioTrackPublication) {
        logMessage(messageText: "Participant \(participant.identity) disabled audio track")
        onDisabledAudioTrack([:])
    }
    
    func failedToSubscribe(toAudioTrack publication: TVIRemoteAudioTrackPublication,
                           error: Error,
                           for participant: TVIRemoteParticipant) {
        logMessage(messageText: "FailedToSubscribe \(publication.trackName) audio track, error = \(String(describing: error))")
        onFailedToSubscribeAudioTrack([:])
    }
    
    func failedToSubscribe(toVideoTrack publication: TVIRemoteVideoTrackPublication,
                           error: Error,
                           for participant: TVIRemoteParticipant) {
        logMessage(messageText: "FailedToSubscribe \(publication.trackName) video track, error = \(String(describing: error))")
        onFailedToSubscribeVideoTrack([:])
    }
}
